/*
 * StbEmu
 * Copyright (c) 2012-2022. Maxim Vasilchuk
 * mvasilchuk@gmail.com
 */

/**
 *
 * @type {Function}
 */

(function (helper) {
    "use strict";

    var XMLHttpRequestProxy = function () {
        return makeProxyObject();
    };

    XMLHttpRequestProxy.UNSENT = 0;
    XMLHttpRequestProxy.OPENED = 1;
    XMLHttpRequestProxy.HEADERS_RECEIVED = 2;
    XMLHttpRequestProxy.LOADING = 3;
    XMLHttpRequestProxy.DONE = 4;

    var requests = {};

    // noinspection JSUnresolvedVariable
    if (window.top && window.top.XMLHttpRequest && window.top.XMLHttpRequest._requests)
        requests = window.top.XMLHttpRequest._requests;
    else { // noinspection JSUnresolvedVariable
        if (window.XMLHttpRequest && window.XMLHttpRequest._requests)
            requests = window.XMLHttpRequest._requests;
    }


    XMLHttpRequestProxy._requests = requests;
    // noinspection JSValidateTypes
    window.XMLHttpRequest = XMLHttpRequestProxy;

    /**
     *
     * @returns {XMLHttpRequest}
     */
    var makeProxyObject = function () {
        var _global_name = ("random_string_" + Math.random()).replace(/\./, '_');
        // noinspection JSUnresolvedFunction
        /**
         * @type {XMLHttpRequest}
         */
        var proxy = helper.createRequest(_global_name, document.location.href);
        proxy.prototype = XMLHttpRequestProxy;

        XMLHttpRequestProxy._requests[_global_name] = proxy;

        /** @type {boolean} */
        /** @type {number} */
        var _timeout = 0;
        /** @type {number} */
        var _readyState = 0;
        /** @type {Object} */
        var _eventListeners = {};
        /** @type {Function} */
        var _onreadystatechange = null;

        /** @type {Function} */
        var _onload = null;
        /** @type {Function} */
        var _onerror = null;
        /** @type {Function} */
        var _onabort = null;

        // noinspection JSUnusedLocalSymbols
        var logCall = function (text) {
        };

        Object.defineProperty(proxy, "_global_name", {
            enumerable: false,
            get: function () {
                return _global_name;
            }
        });

        Object.defineProperty(proxy, "responseText", {
            enumerable: true,
            get: function () {
                logCall("responseText");
                // noinspection JSUnresolvedFunction
                var text = this.getResponseText();

                // Patch for NFPS
                if (text === 'ITV Authorization failed.') {
                    text = 'Authorization failed.';
                }

                logCall(" -> " + text);

                return text;
            },
            set: function (value) {
                logCall('responseText: ' + value);
            }
        });

        Object.defineProperty(proxy, "withCredentials", {
            enumerable: true,
            //writable: true,
            configurable: true,
            get: function () {
                // noinspection JSUnresolvedVariable
                logCall("withCredentials -> " + _withCredentials);
                // noinspection JSUnresolvedFunction
                return this.getWithCredentials();
            },
            set: function (value) {
                // noinspection JSUnresolvedFunction
                this.setWithCredentials(value);
            }
        });

        Object.defineProperty(proxy, "timeout", {
            enumerable: true,
            //writable: true,
            configurable: true,
            get: function () {
                logCall("timeout() -> " + _timeout);
                return _timeout;
            },
            set: function (value) {
                logCall("timeout(" + value + ")");
                _timeout = value;
            }
        });

        Object.defineProperty(proxy, "onreadystatechange", {
            enumerable: true,
            //writable: true,
            configurable: true,
            get: function () {
                logCall("onreadystatechange() -> " + _onreadystatechange);
                return _onreadystatechange;
            },
            set: function (value) {
                logCall("onreadystatechange(" + _onreadystatechange + ")");
                _onreadystatechange = value;
                if (value !== null) {
                    _onreadystatechange.bind(this);
                }
            }
        });

        Object.defineProperty(proxy, "response", {
            enumerable: true,
            get: function () {
                logCall("response");
                return null; //@todo: fix me
            }
        });

        // noinspection JSUnusedLocalSymbols
        Object.defineProperty(proxy, "responseType", {
            enumerable: true,
            get: function () {
                // noinspection JSUnresolvedFunction
                logCall("responseType() -> " + this.getResponseType());
                // noinspection JSUnresolvedFunction
                return this.getResponseType();
            },
            set: function (value) {
            }
        });

        /**
         * {@link http://www.w3.org/TR/XMLHttpRequest2/#the-overridemimetype-method}
         */
        Object.defineProperty(proxy, "overrideMimeType", {
            enumerable: true,
            get: function () {
                return function (mimeType) {
                    try {
                        this.overrideMimeType(mimeType);
                    } catch (e) {
                        console.error(e);
                        return null;
                    }

                }
            }
        });

        Object.defineProperty(proxy, "readyState", {
            enumerable: true,
            get: function () {
                logCall("readyState()");
                if (_readyState === 0) { // noinspection JSUnresolvedFunction
                    _readyState = this.getReadyState();
                }
                logCall(" -> " + _readyState);
                return _readyState;
            },
            /**
             *
             * @param {number} value
             */
            set: function (value) {
                logCall("readyState:set:" + value);
                _readyState = value;
                if (this.onreadystatechange)
                    this.onreadystatechange();
                var progressName = null;
                switch (value) {
                    case -1: {
                        progressName = 'error';
                        if (this.onerror)
                            this.onerror();
                        value = XMLHttpRequest.DONE;
                        scheduleDeleteObject(_global_name);
                        break;
                    }
                    case 0: //XMLHttpRequest.UNSENT
                    {
                        break;
                    }
                    case 1: //XMLHttpRequest.OPENED
                    {
                        progressName = 'loadstart';
                        break;
                    }
                    case 2: //XMLHttpRequest.HEADERS_RECEIVED
                    case 3: //XMLHttpRequest.LOADING
                    {
                        progressName = 'progress';
                        break;
                    }
                    case 4: //XMLHttpRequest.DONE
                    {
                        progressName = 'load';
                        if (this.onload)
                            this.onload();
                        scheduleDeleteObject(_global_name);
                        break;
                    }
                }
                if (progressName !== null) {
                    this.fireProgressEvent(progressName);
                }

                if (_eventListeners[progressName]) {
                    var list = _eventListeners[progressName] || [];
                    if (!list.length) return;
                    for (var index = 0; index < list.length; index++) {
                        try {
                            list[index]();
                        } catch (e) {
                            console.error(e);
                        }
                    }
                }
            }
        });

        var scheduleDeleteObject = function (name) {
            setTimeout(function () {
                if (name in XMLHttpRequestProxy._requests) {
                    delete XMLHttpRequestProxy._requests[name];
                }
            }, 30 * 1000);
        };

        Object.defineProperty(proxy, "responseXML", {
            enumerable: true,
            get: function () {
                logCall("responseXML()");
                /**
                 {@link http://www.w3schools.com/dom/dom_parser.asp}

                 @example
                 <pre>
                 <code>
                 if (window.DOMParser)
                 {
                     parser=new DOMParser();
                     xmlDoc=parser.parseFromString(text,"text/xml");
                 }
                 else // Internet Explorer
                 {
                     xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
                     xmlDoc.async=false;
                     xmlDoc.loadXML(text);
                 }
                 </code>
                 </pre>
                 */

                try {
                    // noinspection JSUnresolvedFunction
                    var text = this.getResponseText();
                    // noinspection JSUnresolvedFunction
                    if (helper.convertAjaxXmlResponse()) {
                        if (text.charAt(0) === '{' || text.charAt(0) === '[') {

                        } else {
                            try {
                                text = (new DOMParser()).parseFromString(text, "text/xml");
                            } catch (e) {
                                console.error(e);
                            }

                        }
                    }

                    logCall(" ->" + text);
                    return text;
                } catch (e) {
                    console.error(e);
                    return '';
                }

            }
        });

        Object.defineProperty(proxy, "status", {
            enumerable: true,
            get: function () {
                // noinspection JSUnresolvedFunction
                logCall("status() -> " + this.getStatus());
                // noinspection JSUnresolvedFunction
                return this.getStatus();
            }
        });

        Object.defineProperty(proxy, "statusText", {
            enumerable: true,
            get: function () {
                // noinspection JSUnresolvedFunction
                logCall("statusText() -> " + this.getStatusText());
                // noinspection JSUnresolvedFunction
                return this.getStatusText();
            }
        });

        Object.defineProperty(proxy, "onload", {
            enumerable: true,
            //writable: true,
            configurable: true,
            get: function () {
                logCall("onload() -> " + _onload);
                return _onload;
            },
            set: function (value) {
                _onload = value;
                if (value !== null) {
                    _onload.bind(this);
                }
            }
        });

        Object.defineProperty(proxy, "onerror", {
            enumerable: true,
            //writable: true,
            configurable: true,
            get: function () {
                return _onerror;
            },
            set: function (value) {
                _onerror = value;
                if (value !== null) {
                    _onerror.bind(this);
                }
            }
        });

        Object.defineProperty(proxy, "onabort", {
            enumerable: true,
            //writable: true,
            configurable: true,
            get: function () {
                return _onabort;
            },
            set: function (value) {
                _onabort = value;
                if (value !== null) {
                    _onabort.bind(this);
                }
            }
        });

        // noinspection JSUndefinedPropertyAssignment
        proxy.fireProgressEvent = function (name) {
            try {
                var event = document.createEvent('ProgressEvent');
                if (event.initEvent) {
                    event.initEvent(name, true, true);
                }
                Object.defineProperty(event, 'lengthComputable', {
                    get: function () {
                        return 1;
                    }
                });
                Object.defineProperty(event, 'loaded', {
                    get: function () {
                        return 1;
                    }
                });
                Object.defineProperty(event, 'total', {
                    get: function () {
                        return 1;
                    }
                });
                //console.log('fireProgressEvent: ' + event);

                //dispatchEvent(event);
            } catch (e) {
                //console.error(e);
            }
        };

        proxy.addEventListener = function (name, callback) {
            //console.log('addEventListener(' + name + ') : ' + this.toString());
            if (!_eventListeners[name])
                _eventListeners[name] = [];
            _eventListeners[name].push(callback.toString());
        };

        return proxy;
    };

    console.log("XMLHttpRequest has been loaded");

})(MyXMLHttpRequestHelper);

MyXMLHttpRequestHelper = undefined;
